/*
	KUMAamp project()
	Copyright (C) Mamiya 2000.
	License:GPL
	---------------------------------------------------------------------------
	GYM sub
	gym.c
*/

#include "in_mdx.h"

/* ANSI/Windows standard headers */
#include <windows.h>

/* Libraries headers */
#if USE_ZLIB
#include <zlib.h>
#endif

/* Project headers */
#include "log.h"
#include "uty.h"

static char templog[MAX_PATH] = "";

/* -------- GYM -------- */

#define GYM_MAGIC 0x584d5947	/* 'GYMX' */

static void GetTitleGYM(char *fn, char *title)
{
	HANDLE hFile = CreateFile(
		fn,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_ARCHIVE |
		FILE_ATTRIBUTE_COMPRESSED |
		FILE_ATTRIBUTE_HIDDEN |
		FILE_ATTRIBUTE_OFFLINE |
		FILE_ATTRIBUTE_READONLY |
		FILE_ATTRIBUTE_SYSTEM |
		FILE_ATTRIBUTE_TEMPORARY,
		NULL);
	if (hFile != INVALID_HANDLE_VALUE)
	{
		DWORD i, len = 0;
		int id = 0;
		char buf[32], *p = title;
		if (ReadFile(hFile, buf, 4, &len, NULL) && len == 4 && GetDwordLE(buf) == GYM_MAGIC)
		{
			SetFilePointer(hFile, 4 + 32, NULL, FILE_BEGIN);
			if (ReadFile(hFile, buf, sizeof(buf), &len, NULL) && buf[0] != '\x00')
			{
				for (i = 0; i < len; i++)
				{
					if (buf[i] == '\x00') break;
					*(p++) = buf[i];
				}
			}
			SetFilePointer(hFile, 4, NULL, FILE_BEGIN);
			if (ReadFile(hFile, buf, sizeof(buf), &len, NULL) && buf[0] != '\x00')
			{
				if (p != title) *(p++) = ' ';
				*(p++) = '[';
				*(p++) = ' ';
				for (i = 0; i < len; i++)
				{
					if (buf[i] == '\x00') break;
					*(p++) = buf[i];
				}
				*(p++) = ' ';
				*(p++) = ']';
			}
			*p = '\0';
			CloseHandle(hFile);
		}
	}
}

static char *LoadGYM(char *infile)
{
#if !USE_ZLIB
	return infile;
#else
	char *gymbuf = NULL, *outbuf = NULL;
	char *ret = infile;
	CleanLOG();
	do
	{
		HANDLE hTemp = INVALID_HANDLE_VALUE;
		char temppath[MAX_PATH], *sp;
		DWORD gymsize, outsize;
		int i, compsize;

		gymbuf = LoadFile(infile, &gymsize);
		if (gymbuf == NULL || gymsize < 4) break;
		if (GetDwordLE(gymbuf) != GYM_MAGIC) break;

		compsize = GetDwordLE(gymbuf + 4 + 32 * 5 + 256 + 4);
		sp = gymbuf + (4 + 32 * 5 + 256 + 4 + 4);
		gymsize -= (4 + 32 * 5 + 256 + 4 + 4);

		if (compsize)
		{
			outsize = compsize;
		}
		else
		{
			outsize = gymsize;
		}

		outbuf = malloc(outsize);
		if (outbuf == NULL) break;

		if (compsize)
		{
			if (Z_OK != uncompress(outbuf, &outsize, sp, gymsize)) break;
		}
		else
			CopyMemory(outbuf, sp, gymsize);

		GetTempPath(MAX_PATH, temppath);
		for (i = 1; hTemp == INVALID_HANDLE_VALUE; i++)
		{
			wsprintf(templog, "%stmp_%d.gym",temppath, i);
			hTemp = CreateFile(
				templog,
				GENERIC_READ | GENERIC_WRITE,
				FILE_SHARE_READ | FILE_SHARE_WRITE/* | FILE_SHARE_DELETE*/,
				NULL,
				CREATE_NEW,
				FILE_ATTRIBUTE_NORMAL/* | FILE_FLAG_DELETE_ON_CLOSE*/,
				NULL
			);
		}
		if (hTemp != INVALID_HANDLE_VALUE)
		{
			WriteFile(hTemp, outbuf, outsize, &gymsize, NULL);
			CloseHandle(hTemp);
			ret = templog;
		}
		else
		{
			templog[0] = '\0';
		}

	} while(0);

	freefile(gymbuf);
	freefile(outbuf);
	if (ret == infile) CleanLOG();
	return ret;
#endif
}

static int GetTimeSubGYM(char *p, int len)
{
	int clock = 0;
	int i = 0;
	while (i < len)
	{
		switch (p[i])
		{
			case 0x00:
				clock++;
				i += 1;
				break;
			case 0x01:	/* YM2412(m) */
			case 0x02:	/* YM2412(s) */
				i += 3;
				break;
			case 0x03:	/* SN76496 */
				i += 2;
				break;
			default:
				return clock * 100 / 6;
		}
	}
	return clock * 100 / 6;
}

static void GetTimeGYM(char *fn, int *time_in_ms)
{
#if !USE_ZLIB
	return;
#else
	char *gymbuf = NULL, *outbuf = NULL;
	do
	{
		char *sp;
		DWORD gymsize, outsize;
		int compsize;

		gymbuf = LoadFile(fn, &gymsize);
		if (gymbuf == NULL || gymsize < 4) break;
		if (GetDwordLE(gymbuf) != GYM_MAGIC)
		{
			*time_in_ms = GetTimeSubGYM(gymbuf, gymsize);
			break;
		}

		compsize = GetDwordLE(gymbuf + 4 + 32 * 5 + 256 + 4);
		sp = gymbuf + (4 + 32 * 5 + 256 + 4 + 4);
		gymsize -= (4 + 32 * 5 + 256 + 4 + 4);

		if (compsize)
		{
			outbuf = malloc(compsize);
			if (outbuf == NULL) break;
			if (Z_OK != uncompress(outbuf, &outsize, sp, gymsize)) break;
			*time_in_ms = GetTimeSubGYM(outbuf, outsize);
		}
		else
		{
			*time_in_ms = GetTimeSubGYM(sp, gymsize);
		}

	} while(0);

	freefile(gymbuf);
	freefile(outbuf);
	return;
#endif
}

static void GetTimeCYM(char *fn, int *time_in_ms)
{
	return;
}

/* -------- LOGS -------- */

void CleanLOG(void)
{
	if (templog[0] != '\0')
	{
		DeleteFile(templog);
		templog[0] = '\0';
	}
}

void GetTimeLOG(char *fn, int *time_in_ms)
{
	char ext[32];
	GetPathExt(ext, fn);
	if (time_in_ms == NULL) return;
	else if (MatchExt(ext, "GYM")) GetTimeGYM(fn, time_in_ms);
#if 0
	else if (MatchExt(ext, "CYM")) GetTimeCYM(fn, time_in_ms);
	else if (MatchExt(ext, "X1F")) GetTimeX1F(fn, time_in_ms);
	else if (MatchExt(ext, "MYM")) GetTimeMYM(fn, time_in_ms);
#endif
}

void GetTitleLOG(char *fn, char *title)
{
	char ext[32];
	GetPathExt(ext, fn);
	if (title == NULL) return;
	else if (MatchExt(ext, "GYM")) GetTitleGYM(fn, title);

}

char *LoadLOG(char *infile)
{
	char ext[32];
	GetPathExt(ext, infile);
	if (MatchExt(ext, "GYM")) infile = LoadGYM(infile);
	return infile;
}
